-------------------------------------------------------------------------------
-- Dark Soil by Lockslap, US-Bleeding Hollow
-- <Borderline Amazing>, http://ba-guild.com
-------------------------------------------------------------------------------
DarkSoil = LibStub("AceAddon-3.0"):NewAddon("Dark Soil", "AceEvent-3.0", "AceConsole-3.0")

-- libraries
local L = LibStub("AceLocale-3.0"):GetLocale("Dark Soil", false)
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

-- globals
DarkSoil.name = L["DarkSoil"]
DarkSoil.title = L["Dark Soil"]
DarkSoil.version = GetAddOnMetadata("DarkSoil", "Version")

-- strings and the like we'll use later
local waypoints = {}
local enabledString = ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString = ("|cffff0000%s|r"):format(L["Disabled"])

local repStandings 		= {
	[1]	= L["Hated"],
	[2]	= L["Hostile"],
	[3]	= L["Unfriendly"],
	[4]	= L["Neutral"],
	[5]	= L["Friendly"],
	[6]	= L["Honored"],
	[7]	= L["Revered"],
	[8]	= L["Exalted"],
}

local friends = {
	1277,	-- Chee Chee
	1275,	-- Ella
	1283,	-- Farmer Fung
	1282,	-- Fish Fellreed
	1281,	-- Gina Mudclaw
	1279,	-- Haohan Mudclaw
	1273,	-- Jogu the Drunk
	1276,	-- Old Hillpaw
	1278,	-- Sho
	1280,	-- Tina Mudclaw
}

-- options and all that jazz
local db, defaults = {}, {
	char = {
		enabled = true,
		notify = false,
		bestFriend = true,
		show = {
			minimap = false,
			worldmap = false,
			arrow = true,
		},
	},
}
local fields = {"Title", "Version", "Author", "X-Category", "X-License", "X-Email", "Email", "eMail", "X-Website", "X-Credits", "X-Localizations", "X-Donate", "X-Paypal"}
local fieldName = "|cff9382c9%s:|r"
local options = {
	type = "group",
	args = {
		general = {
			type = "group",
			name = DarkSoil.name,
			get = function(info) return db[info[#info]] end,
			set = function(info, value)
				db[info[#info]] = value
				DarkSoil:RemoveWaypoints()
				if db.enabled then
					DarkSoil:LoadWaypoints()
				end
			end,
			args = {
				header = {
					type = "header",
					order = 1,
					name = (L["Version: |cff99ff33%s|r"]):format(DarkSoil.version),
					width = "full",
				},
				description = {
					type = "description",
					order = 2,
					name = L["A simple addon to manage the known spawn points for Dark Soil objects."],
					width = "full",
				},
				enabled = {
					type = "toggle",
					order = 3,
					name = L["Enabled"],
				},
				notify = {
					type = "toggle",
					order = 4,
					name = L["Display Notifications"],
					disabled = function() return not db.enabled end,
				},
				bestFriend = {
					type = "toggle",
					order = 5,
					name = L["Hide Best Friends"],
					desc = L["Hide best friends from the friends list.  This will be ignored if you use the |cff00ff96all|r parameter."],
				},
				show = {
					type = "group",
					order = 99,
					guiInline = true,
					name = L["Show On..."],
					disabled = function() return not db.enabled end,
					get = function(info) return db.show[info[#info]] end,
					set = function(info, value)
						db.show[info[#info]] = value
						DarkSoil:RemoveWaypoints()
						if db.enabled then
							DarkSoil:LoadWaypoints()
						end
					end,
					args = {
						arrow = {
							type = "toggle",
							order = 1,
							name = L["Crazy Arrow"],
							desc = L["Show waypoint on TomTom's crazy arrow."],
						},
						minimap = {
							type = "toggle",
							order = 2,
							name = L["Minimap"],
							desc = L["Show waypoint on the minimap."],
						},
						worldmap = {
							type = "toggle",
							order = 3,
							name = L["World Map"],
							desc = L["Show waypoint on the world map."],
						},
					},
				},
			},
		},
		about = {
			type = "group",
			name = L["About"],
			args = {},
		},
	},
}

local function RepStatus(id, friendship)
	if not friendship then
		local name, _, standing, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(id)
		barValue = barValue - barMin
		barMax = barMax - barMin
		return name, barValue, barMax, standing, math.floor(barValue / barMax * 100)
	else
		local friendID, friendRep, friendMaxRep, friendText, _, friendTextLevel, friendThresh = GetFriendshipReputationByID(id)
		if friendID ~= nil then
			local barValue = friendRep - friendThresh
			local barMax = math.min(friendMaxRep - friendThresh, 8400)
			local barMin = 0
			return friendText, barValue, barMax, friendTextLevel, math.floor(barValue / barMax * 100)
		else
			return false
		end
	end
end

local function FindFriend(name)
	if not name then return false end
	for id, friend in pairs(DarkSoil.list) do
		if name:lower():gsub(" ", "") == friend:lower():gsub(" ", "") then
			return id, friend
		elseif type(name) == "number" and id == name then
			return id, friend
		end
	end
	return false
end

function DarkSoil:InArray(value)
	if #waypoints == 0 then return false end
	for i = 1, #waypoints do
		if waypoints[i] == value then
			return i
		end
	end
	return false
end

function DarkSoil:LoadWaypoints()
	local zone = GetCurrentMapAreaID()
	local zoneName = GetMapNameByID(zone)
	
	if db.enabled and GetCurrentMapContinent() == 6 and self.coordinates[zone] then
		for _, point in pairs(self.coordinates[zone]) do
			local opts = {
				title = (L["Dark Soil\n%s"]):format(zoneName),
				persistent = false,
				minimap = db.show.minimap,
				world = db.show.worldmap,
				crazy = db.show.arrow,
				cleardistance = 0,
				arrivaldistance = 0,
				callbacks = {}
			}
			opts.callbacks = TomTom:DefaultCallbacks(opts)
			opts.callbacks.distance[15] = function(event, uid, range, distance, lastDistance)
				if lastDistance > distance then
					local inArray = DarkSoil:InArray(uid)
					TomTom:RemoveWaypoint(uid)
					if inArray ~= false then
						waypoints[inArray] = nil
					end
					
					if db.notfiy and #waypoints == 0 then
						self:Print((L["All waypoints for |cff9382c9%s|r are completed. Type |cff9382c9/darksoil reload|r to reload."]):format(zoneName))
					end
				end
			end
			waypoints[#waypoints + 1] = TomTom:AddMFWaypoint(zone, 0, point.x / 100, point.y / 100, opts)
		end
		
		if db.notify then
			self:Print((L["Loaded |cff9382c9%d|r waypoints for |cff9382c9%s|r."]):format(#self.coordinates[zone], zoneName))
		end
	elseif not db.enabled then
		self:RemoveWaypoints()
	end
end

function DarkSoil:RemoveWaypoints()
	if #waypoints == 0 then return end
	for i = 1, #waypoints do
		if waypoints[i] then
			TomTom:RemoveWaypoint(waypoints[i])
		end
	end
end

function DarkSoil:ZONE_CHANGED_NEW_AREA(event, ...)
	self:LoadWaypoints()
end
DarkSoil:RegisterEvent("ZONE_CHANGED_NEW_AREA")

function DarkSoil:ChatCommand(args)
	local command, value = self:GetArgs(args, 2)
	
	if command == "reload" then
		self:RemoveWaypoints()
		self:LoadWaypoints()
		
	elseif command == "config" then
		InterfaceOptionsFrame_OpenToCategory(self.frames.general)
	
	elseif command == "tillers" then
		local repString = "%s |cff9382c9%d/%d %s (%d%%)|r %s |cff9382c9%s|r."
		local name, value, max, standing, percent = RepStatus(1272)
		self:Print(repString:format(L["You are"], value, max, repStandings[standing], percent, L["with"], name))
	
	-- single friend
	elseif command == "friend" then
		if not value then
			self:Print(L["You must specify a friend's name (in quotes), or ID."])
		else
			local id, name = FindFriend(value)
			if not id or not name then
				self:Print((L["Friend |cff9382c9%s|r not found."]):format(value))
			else
				local _, _, standingID, _, _, _, _, _, _, _, _, _, _ = GetFactionInfoByID(id)
				local _, value, max, standing, percent = RepStatus(id, true)
				local color = FACTION_BAR_COLORS[standingID]
				local hex = ("%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
				self:Print((L["You are |cff%s%d/%d %s (%d%%)|r with |cff9382c9%s|r."]):format(hex, value, max, standing, percent, name))
			end
		end
	
	-- all friends
	elseif command == "friends" then
		local start = value == "next" and 6 or 1
		local stop = value == "next" and 10 or 5
		local hideBestFriends = db.bestFriend
		if value == "all" then
			start = 1
			stop = 10
			hideBestFriends = false
		end
		for i = start, stop do
			local name, _, standingID, _, _, _, _, _, _, _, _, _, _ = GetFactionInfoByID(friends[i])
			local _, value, max, standing, percent = RepStatus(friends[i], true)
			if not hideBestFriends or standingID ~= 8 then
				local color = FACTION_BAR_COLORS[standingID]
				local hex = ("%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
				self:Print((L["You are |cff%s%d/%d %s (%d%%)|r with |cff9382c9%s|r."]):format(hex, value, max, standing, percent, name))
			end
		end
		if not value or (value ~= "next" and value ~= "all") then
			self:Print(L["Type |cff9382c9/darksoil friends next|r to see the other half."])
		end
	
	elseif command == "toggle" or not command then
		db.enabled = not db.enabled
		self:Print((L["Waypoints have been %s"]):format(db.enabled == true and enabledString or disabledString))
		self:RemoveWaypoints()
		if db.enabled then
			self:LoadWaypoints()
		end
	end
end

function DarkSoil:OnEnable()
	self:RegisterChatCommand("darksoil", "ChatCommand")
	self:RegisterChatCommand("ds", "ChatCommand")
	
	if db.enabled then
		self:LoadWaypoints()
	end
end

function DarkSoil:OnDisable()
	self:UnregisterChatCommand("darksoil")
	self:UnregisterChatCommand("ds")
end

function DarkSoil:OnInitialize()
	-- load the config
	self.db = LibStub("AceDB-3.0"):New("DarkSoilDB", defaults)
	db = self.db.char
	
	-- next setup tomtom
	TomTom.profile.arrow.enable = true
	TomTom.profile.arrow.arrival = 5
	TomTom.profile.arrow.setclosest = true
	
	-- setup the options frame
	local i = 1
	for _, field in pairs(fields) do
		local value = GetAddOnMetadata(self.name, field)
		if value then
			options.args.about.args[field] = {
				type = "description",
				order = i,
				width = "normal",
				name = fieldName:format(field:gsub("X%-", "")),
			}
			i = i + 1
			options.args.about.args[("%s_value"):format(field)] = {
				type = "description",
				order = i,
				width = "double",
				name = value,
			}
			i = i + 1
		end
	end
	
	-- now setup the config window
	AceConfigRegistry:RegisterOptionsTable(self.title, options)
	self.frames = {}
	self.frames.general = AceConfigDialog:AddToBlizOptions(self.title, nil, nil, "general")
	self.frames.about = AceConfigDialog:AddToBlizOptions(self.title, L["About"], self.title, "about")
	
	-- load the tillers NPC names
	for i = 1, #friends do
		local name, _, _, _, _, _, _, _, _, _, _, _, _ = GetFactionInfoByID(friends[i])
		self.list[friends[i]] = name
	end
end